//=============================================================================
// Microcontroller Serial Ʈ  API Լ
// [serial.c]
//=============================================================================
#include "system.h"
#include "serial.h"

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
//------------------------------------------------------------------------------
// UART Defination
//------------------------------------------------------------------------------
#define BAUDRATE_VAL_9600					9600
#define BAUDRATE_VAL_19200					19200
#define BAUDRATE_VAL_14400					14400
#define BAUDRATE_VAL_28800					28800
#define BAUDRATE_VAL_38400					38400
#define BAUDRATE_VAL_57600					57600
#define BAUDRATE_VAL_76800					76800
#define BAUDRATE_VAL_115200					115200
#define BAUDRATE_VAL_DEFAULT				BAUDRATE_VAL_57600


#ifdef _UART0_ENABLE

//------------------------------------------------------------------------------
// Serial Prot0 Utility Fuction Routine
void U0_Init(unsigned char baud)
{
	U0_SetBaud(baud);
}

void U0_SetBaud(unsigned char baud)
{
	unsigned int baud_rate = 0;

	switch (baud)
	{
		case BAUD_9600:
			baud_rate = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			baud_rate = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			baud_rate = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			baud_rate = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			baud_rate = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			baud_rate = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			baud_rate = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			baud_rate = BAUDRATE_VAL_115200;
			break;		
		default :
			baud_rate = BAUDRATE_VAL_DEFAULT;			
			break;
	}	

    //
    // Enable the UART and GPIO modules.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);

    //
    // Make the UART pins be peripheral controlled.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
	

    //
    // Configure the UART 
    //
    UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), baud_rate,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));	
}

unsigned char U0_GetByte(void)
{
	return UARTCharGet(UART0_BASE);	
}

void U0_PutByte(unsigned char Data)
{
	//UARTCharPutNonBlocking(UART0_BASE, Data);
	UARTCharPut(UART0_BASE, Data);
}

void U0_PutBuf(unsigned char *str, unsigned char len)
{
	unsigned char i =0;
	
    for(i=0;i<len;i++)
    {
        U0_PutByte(*(str+i));
    }
}

void U0_PutStr(char *str)
{
    while(*str != 0)
    {
        U0_PutByte(*str);
        str++;
    }
}

unsigned char U0_IsGetByte(void)
{
	return 0;
}

void U0_printf(char *fmt,...)
{
    va_list ap;
    char string[80];

    va_start(ap,fmt);
    vsprintf(string,fmt,ap);
    U0_PutStr(string);
    va_end(ap);
}

void U0_SetInt(void)
{
    // Enable the UART interrupt.
    //
    IntEnable(INT_UART0);
    UARTIntEnable(UART0_BASE, UART_INT_RX | UART_INT_RT);
}
//------------------------------------------------------------------------------
#endif	//end UART0 ENABLE


#ifdef _UART1_ENABLE
//------------------------------------------------------------------------------
// Serial Prot1 Utility Fuction Routine
void U0_Init(unsigned char baud)
{
	U1_SetBaud(baud);
	
	USART_Cmd(USART2, ENABLE);
}

void U1_SetBaud(unsigned char baud)
{
	unsigned int baud_rate = 0;
	GPIO_InitTypeDef  GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;

	switch (baud)
	{
		case BAUD_9600:
			baud_rate = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			baud_rate = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			baud_rate = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			baud_rate = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			baud_rate = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			baud_rate = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			baud_rate = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			baud_rate = BAUDRATE_VAL_115200;
			break;		
		default :
			baud_rate = BAUDRATE_VAL_DEFAULT;			
			break;
	}	

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART2 | RCC_APB2Periph_GPIOA, ENABLE);

	/* Configure USART2 Rx (PA3) as input floating                             */
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART2 Tx (PA2) as alternate function push-pull                */
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	USART_InitStructure.USART_BaudRate = baud_rate;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No ;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	USART_Init(USART2, &USART_InitStructure);
}

unsigned char U1_GetByte(void)
{
	while (USART_GetFlagStatus(USART2, USART_FLAG_RXNE) == RESET);
	return (unsigned char)USART2->DR;;	
}

void U1_PutByte(unsigned char Data)
{
	while(USART_GetFlagStatus(USART2, USART_FLAG_TXE) == RESET);
	USART_SendData(USART2, Data);
}

void U1_PutBuf(unsigned char *str, unsigned char len)
{
	unsigned char i =0;
	
    for(i=0;i<len;i++)
    {
        U1_PutByte(*(str+i));
    }
}

void U1_PutStr(unsigned char *str)
{
    while(*str != 0)
    {
        U1_PutByte(*str);
        str++;
    }
}

unsigned char U1_IsReceiveData(void)
{
	return u1_IsReceiveData();
}

void U1_SetInt(void)
{
}
//------------------------------------------------------------------------------
#endif	//end _UART1_ENABLE



#ifdef _UART2_ENABLE
//-----------------------------------------------------------------------------
// USART Fuction Declaration
unsigned char U2_IsGetByte(void);
void U2_Init(unsigned char baud);
void U2_SetBaud(unsigned char baud);
unsigned char U2_GetByte(void);
void U2_PutByte(unsigned char Data);
void U2_PutStr(char *str);
void U2_PutBuf(unsigned char *str, unsigned char len);
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Serial Prot1 Utility Fuction Routine
void U2_Init(unsigned char baud)
{
	U2_SetBaud(baud);
	
	u2_hal_init();
}

void U2_SetBaud(unsigned char baud)
{
	u2_baudrate(baud);
}

unsigned char U2_GetByte(void)
{
	u2_WaitForReceive();
	
	return u2_RxData();
}

void U2_PutByte(unsigned char Data)
{
	u2_TxData(Data);
	u2_WaitForSend();
}

void U2_PutBuf(unsigned char *str, unsigned char len)
{
	unsigned char i =0;
	
    for(i=0;i<len;i++)
    {
        U2_PutByte(*(str+i));
    }
}

void U2_PutStr(char *str)
{
    while(*str != 0)
    {
        u2_TxData(*str);
        str++;
    }
}

unsigned char U1_IsGetByte(void)
{
	return (USART_GetFlagStatus(USART2, USART_FLAG_RXNE) == RESET);
}

void U1_printf(char *fmt,...)
{
    va_list ap;
    char string[128];

    va_start(ap,fmt);
    vsprintf(string,fmt,ap);
    U1_PutStr(string);
    va_end(ap);
}
//------------------------------------------------------------------------------
#endif	//end _UART2_ENABLE
